"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _grpc = require("grpc");

var _grpc2 = _interopRequireDefault(_grpc);

var _dataloader = require("dataloader");

var _dataloader2 = _interopRequireDefault(_dataloader);

var _ = require("./");

var _stringhelpers = require("./stringhelpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
	var domainService = _ref.domainService,
	    services = _ref.services;

	var serviceMethodGenerators = {};
	var contextDomainEntitys = {};

	var getContextDomainEntity = function getContextDomainEntity(context, domainEntity) {
		return function (model) {
			var _this = this;

			if (typeof domainEntity.fields !== "function") {
				throw new Error("DomainEntity " + domainEntity.name + " fields needs to be function");
			}
			var fields = domainEntity.fields();
			Object.keys(fields).forEach(function (fieldName) {
				var field = fields[fieldName];
				if (field.serviceMethod) {
					var args = {};

					if (field.inhertedArgs) {
						Object.keys(field.inhertedArgs).forEach(function (argName) {
							args[field.inhertedArgs[argName]] = model["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(argName)]();
						});
					}

					_this[fieldName] = function (props) {
						var newArgs = _extends({}, args);
						if (field.args) {
							Object.keys(field.args).forEach(function (argName) {
								newArgs[argName] = props[argName];
							});
						}
						return new Promise(function (resolve, reject) {
							if (!context[field.serviceMethod] || typeof context[field.serviceMethod] !== "function") {
								throw new Error("DomainEntity " + domainEntity.name + " " + field.serviceMethod + " is not function");
							}
							context[field.serviceMethod](newArgs).then(function (res) {
								if (field.returnField) {
									resolve(res[field.returnField]);
								} else {
									resolve(res);
								}
							}).catch(function (err) {
								reject(err);
							});
						});
					};
				} else {
					if (field.type instanceof _.DomainEntity) {
						if (typeof model["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(fieldName)] === "function") {
							Object.defineProperty(_this, fieldName, {
								get: function get() {
									return new (getContextDomainEntity(context, field.type))(model["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(fieldName)]());
								}
							});
						} else {
							console.warn(domainEntity.name + " model does not have domainentity get method get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(fieldName));
						}
					} else if (field.type instanceof _.ListType) {
						if (typeof model["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(fieldName) + "List"] === "function") {
							Object.defineProperty(_this, fieldName, {
								get: function get() {
									var NewType = getContextDomainEntity(context, field.type.type);
									return model["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(fieldName) + "List"]().map(function (p) {
										return new NewType(p);
									});
								}
							});
						} else {
							console.warn(domainEntity.name + " model does not have list get method get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(fieldName) + "List");
						}
					} else {
						if (typeof model["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(fieldName)] === "function") {
							Object.defineProperty(_this, fieldName, {
								get: function get() {
									return model["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(fieldName)]();
								}
							});
						} else {
							console.warn(domainEntity.name + " model does not have get method get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(fieldName));
						}
					}
				}
			});
		};
	};

	Object.keys(domainService.methods).forEach(function (methodName) {
		var method = domainService.methods[methodName];
		serviceMethodGenerators[methodName] = CreateServiceMethodGenerator({
			serviceMethodName: method.serviceMethodName || methodName,
			method: method,
			messages: domainService.messages,
			getContextDomainEntity: getContextDomainEntity
		});
	});

	return function (config) {

		if (!config) {}
		var ip = config.ip,
		    port = config.port;

		var client = new domainService.services[(0, _stringhelpers.capitalizeFirstLetter)(domainService.name) + "Client"](ip + ":" + port, _grpc2.default.credentials.createInsecure());

		var context = {};
		Object.keys(serviceMethodGenerators).forEach(function (methodName) {
			context[methodName] = serviceMethodGenerators[methodName](context, client);
		});
		return context;
	};
};

var CreateServiceMethodGenerator = function CreateServiceMethodGenerator(_ref2) {
	var serviceMethodName = _ref2.serviceMethodName,
	    method = _ref2.method,
	    messages = _ref2.messages,
	    getContextDomainEntity = _ref2.getContextDomainEntity;

	return function (context, client) {
		if (client[serviceMethodName] && typeof client[serviceMethodName] === "function") {
			if (method.dataLoader) {
				var dataLoader = new _dataloader2.default(function (keys) {
					return new Promise(function (resolve, reject) {
						var req = void 0;
						var requestMessageName = method.requestTypeName;
						if (requestMessageName) {
							if (!messages[method.requestTypeName] && messages[method.requestTypeName] !== "function") {
								throw new Error("There is no messagetype " + requestMessageName);
							} else {
								req = new messages[requestMessageName]();
							}
						} else {
							requestMessageName = (0, _stringhelpers.capitalizeFirstLetter)(serviceMethodName) + "Request";
							if (messages[requestMessageName] && typeof messages[requestMessageName] === "function") {
								req = new messages[requestMessageName]();
							} else {
								throw new Error("There is no messagetype " + requestMessageName);
							}
						}
						if (!req["set" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(method.dataLoader.serviceTypeName) + "List"] && typeof req["set" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(method.dataLoader.serviceTypeName) + "List"] !== "function") {
							throw new Error(requestMessageName + " type does not have property " + method.dataLoader.serviceTypeName);
						}

						req["set" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(method.dataLoader.serviceTypeName) + "List"](keys);
						client[serviceMethodName](req, function (err, res) {
							if (err) {
								reject(err);
							} else {
								resolve(res["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(method.dataLoader.returnField) + "List"]().map(function (p) {
									if (p.getId() > 0) {
										return new (getContextDomainEntity(context, method.type))(p);
									} else {
										return null;
									}
								}));
							}
						});
					});
				});
				return function (props) {
					return dataLoader.load(props[method.dataLoader.name]);
				};
			} else {
				return function (props) {
					return new Promise(function (resolve, reject) {
						var req = void 0;
						if (method.requestTypeName) {
							if (!messages[method.requestTypeName] && messages[method.requestTypeName] !== "function") {
								throw new Error("There is no messagetype " + method.requestTypeName);
							} else {
								req = new messages[method.requestTypeName]();
							}
						} else {
							var requestMessageName = (0, _stringhelpers.capitalizeFirstLetter)(serviceMethodName) + "Request";
							if (messages[requestMessageName] && typeof messages[requestMessageName] === "function") {
								req = new messages[requestMessageName]();
							} else {
								throw new Error("There is no messagetype " + requestMessageName);
							}
						}
						if (props) {
							Object.keys(method.args).forEach(function (argName) {
								var arg = method.args[argName];
								if (arg.type instanceof _.ListType) {
									if (req["set" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(argName) + "List"]) {
										req["set" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(argName) + "List"](props[argName]);
									} else {
										console.error(serviceMethodName, "has no method set" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(argName) + "List");
									}
								} else {
									if (req["set" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(argName)]) {
										req["set" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(argName)](props[argName]);
									} else {
										console.error(serviceMethodName, "has no method set" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(argName));
									}
								}
							});
						}
						client[serviceMethodName](req, function (err, res) {
							if (err) {
								reject(err);
							} else {
								if (method.returnField) {
									if (method.type instanceof _.ListType) {
										if (res["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(method.returnField) + "List"]) {
											resolve(res["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(method.returnField) + "List"]().map(function (p) {
												return new (getContextDomainEntity(context, method.type.type))(p);
											}));
										}
									} else {
										if (res["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(method.returnField)]) {
											resolve(new (getContextDomainEntity(context, method.type))(res["get" + (0, _stringhelpers.capitalizeOnlyFirstLetter)(method.returnField)]()));
										}
									}
								} else {
									resolve(new (getContextDomainEntity(context, method.type))(res));
								}
							}
						});
					});
				};
			}
		} else {
			throw new Error("Grpc client does not have method " + serviceMethodName);
		}
	};
};